/*
 * $Id: timer.prg 693 2011-09-01 20:53:19Z l3wonsf $
 */

/*
 * HMG Source Code
 *
 * Copyright 2002-2010 Roberto Lopez <mail.box.hmg@gmail.com>
 * http://sites.google.com/site/hmgweb/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or( at your option ) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License FOR more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this software; see the file COPYING. IF not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA( or
 * visit the web site http://www.gnu.org/ ).
 *
 * As a special exception, you have permission FOR additional uses of the text
 * contained in this release of HMG.
 *
 * The exception is that, IF you link the HMG library with other
 * files to produce an executable, this does not by itself cause the resulting
 * executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of linking the
 * HMG library code into it.
 */

#include "hbclass.ch"
#include "common.ch"
#include "hbqtgui.ch"

/*==============================================================================
   TIMER class
==============================================================================*/
CLASS TIMER FROM CONTROL

   DATA cClass                                    INIT   "TIMER"

// data: please preserve alphabetic order.
   DATA lStarted                                  INIT   NIL         PROTECTED   // please see Enabled method before restore to .T.
   DATA nInterval                                 INIT   NIL         PROTECTED   // please see Interval method before restore to 0
   DATA bOnInterval                               INIT   NIL         PROTECTED
   DATA lSingleShot                               INIT   .F.         PROTECTED

// Methods: please preserve alphabetic order. Methods "New" and "Create" must be the first if present
   METHOD New
   METHOD Create
   METHOD Connect
   METHOD Disconnect
   METHOD IsActive

// Properties & Events: please preserve alphabetic order.
   METHOD Enabled                                 SETGET
   METHOD Interval                                SETGET
   METHOD OneShot                                 SETGET
   METHOD OnInterval                              SETGET

// Connected events handling: please preserve alphabetic order.
   METHOD ZonIntervalDo
   METHOD ZonCloseDo

ENDCLASS

/*-----------------------------------Methods----------------------------------*/

/*..............................................................................
   New
..............................................................................*/
METHOD New( cName, oParent ) CLASS TIMER

   Super:New( cName, oParent )

   // Create QT Object
   ::oQTObject := QTimer( ::oParent:QtObject )

   RETURN Self

/*..............................................................................
   Create
..............................................................................*/
METHOD Create() CLASS TIMER

   ::lCreated  := .T.
   
   ::Connect()

   ::oQTObject:Start( ::Interval() )

   RETURN Self

/*..............................................................................
   Connect
..............................................................................*/
METHOD Connect() CLASS TIMER

   ::oQtObject:connect( QEvent_Close,           { || ::ZonCloseDo() } )
   ::oQTObject:connect( "timeout()",            { || ::ZonIntervalDo() } )

   RETURN Self

/*..............................................................................
   Disconnect
..............................................................................*/
METHOD Disconnect() CLASS TIMER

   ::oQtObject:disconnect( QEvent_Close )
   ::oQTObject:disconnect( "timeout()" )

   RETURN Self

/*..............................................................................
   IsActive
..............................................................................*/
METHOD IsActive() CLASS TIMER

   RETURN ::Enabled()

/*-------------------------------Properties & Events--------------------------*/

/*..............................................................................
   Enabled
..............................................................................*/
METHOD Enabled( lValue ) CLASS TIMER

   IF PCOUNT() == 0
      ::lStarted  := IIF( hb_IsNil( ::lStarted ) == .T., ::oQTObject:isActive(), ::lStarted )
      RETURN ::lStarted
   ELSEIF hb_IsLogical( lValue )
      ::lStarted := lValue
      IF ::lStarted == .T.
         IF ::oQTObject:isActive() == .F.
           ::oQTObject:start()
         ENDIF
      ELSE
         IF ::oQTObject:isActive() == .T.
            ::oQTObject:stop()
         ENDIF
      ENDIF
   ENDIF

   RETURN ::lStarted       // this is out of standard method with SETGET

/*..............................................................................
   Interval
..............................................................................*/
METHOD Interval( nValue ) CLASS TIMER

   IF PCOUNT() == 0
      ::nInterval := IIF( hb_IsNil( ::nInterval ) == .T., ::oQTObject:interval(), ::nInterval )
      RETURN ::nInterval
   ELSEIF PCOUNT() == 1
      ::nInterval := nValue
      ::oQTObject:setInterval( ::nInterval )
   ENDIF

   RETURN NIL

/*..............................................................................
   OneShot
..............................................................................*/
METHOD OneShot( lValue ) CLASS TIMER

   IF hb_IsLogical( lValue )
      ::oQTObject:setSingleShot( lValue )
      ::lSingleShot := lValue
   ENDIF

   RETURN ::lSingleShot

/*..............................................................................
   OnInterval
..............................................................................*/
METHOD OnInterval( bValue ) CLASS TIMER

   IF pcount() == 0
      RETURN ::bOnInterval
   ELSEIF pcount() == 1
      ::bOnInterval := bValue
   ENDIF

   RETURN NIL

/*--------------------------Connected events handling-------------------------*/

/*..............................................................................
   ZonCloseDo
..............................................................................*/
METHOD ZonCloseDo() CLASS TIMER

   LOCAL lAccept       // to pacified Qt accept or reject signal

// This is an example how can we do within this event
//   IF hb_IsBlock( ::bOnRelease ) == .T.
//      lAccept := EVAL( ::bOnRelease )
//   ENDIF
   lAccept := IIF( hb_IsLogical( lAccept ) == .T., lAccept, .T. ) //by default in absence of a logical value returned by EVAL, is accepted

RETURN lAccept

/*..............................................................................
   ZonInterval
..............................................................................*/
METHOD ZonIntervalDo() CLASS TIMER

   LOCAL lAccept       // to pacified Qt accept or reject signal

   IF hb_IsBlock( ::bOnInterval ) == .T.
      lAccept := EVAL( ::bOnInterval )
   ENDIF
   lAccept := IIF( hb_IsLogical( lAccept ) == .T., lAccept, .T. ) //by default in absence of a logical value returned by EVAL, is accepted

RETURN lAccept
